/*
     DS18B20   
       , 
        .
  ..
 http://mega-avr.com.ua
 http://mega-avr.tk
 2017 .
*/


#include <delay.h>

#define DS18B20_9BIT    0b00011111
#define DS18B20_10BIT   0b00111111
#define DS18B20_11BIT   0b01011111
#define DS18B20_12BIT   0b01111111

#define sei()     #asm ("sei")
#define cli()     #asm ("cli")

#define ERROR_CRC    9999
#define ERROR_TMP   -9999

volatile unsigned char pin=0;    //          

#define W1_PORT PORTB
#define W1_DDR  DDRB
#define W1_PIN  PINB
#define W1_BIT  pin

void init_w1_ds(signed char th, signed char tl, unsigned char raz); // init   ,     
unsigned char w1_find();//      
void w1_sendcmd(unsigned char cmd);//     1-wire
unsigned char w1_receive_byte();//      1-wire
int temp_18b20(unsigned char pinn);//     18b20    ( 0  1 )
unsigned char crc8(unsigned char d,unsigned char crc_s); //  
void ds18b20_convert_t_rom(); //    
int ds18b20_result_t_rom();  //    
unsigned char w1_receive_bit(); // //   bit   1-wire
void ds18b20_pin (unsigned char pin_ds);  //              
unsigned char ds18b20_convert_ok(unsigned char pin_ds); //  0      1       
float Ftemp_18b20(unsigned char pinn); //     18b20      float 0.0

void init_w1_ds(signed char th, signed char tl, unsigned char raz)
{
 
   if(w1_find()==1)//    
 {  
   w1_sendcmd(0xcc);// ROM ,  ,        
  
   w1_sendcmd(0x4E); 
   w1_sendcmd(th); 
   w1_sendcmd(tl); 
   w1_sendcmd(raz);  
   w1_find();//  Presence  Reset
 }
 
}

void ds18b20_pin(unsigned char pin_ds)
{
  pin = pin_ds;
}

unsigned char ds18b20_convert_ok(unsigned char pin_ds)
{
  if((W1_PIN & (1<<pin_ds)) == 0x00)
  return 0;
  else
  return 1;
}

//      
unsigned char w1_find()
{
 unsigned char device=0; 
 
 W1_DDR |= (1<<W1_BIT);
 W1_PORT&=~(1<<W1_BIT);
 delay_us(480); 
 W1_DDR &= ~(1<<W1_BIT);
 W1_PORT&= ~(1<<W1_BIT);
 delay_us(70);

 if((W1_PIN & (1<<W1_BIT)) ==0x00)
 device = 1;  
 else
 device = 0;   
 delay_us(410); 
 return device;
}

//     1-wire
void w1_sendcmd(unsigned char cmd)
{
 unsigned char i = 0;
 for(i = 0; i < 8; i++)
 {
  if((cmd & (1<<i)) == 1<<i)      
   {
   W1_DDR |= 1<<W1_BIT; 
   W1_PORT&= ~(1<<W1_BIT);
   delay_us(15);
   W1_DDR &= ~(1<<W1_BIT); 
   W1_PORT&= ~(1<<W1_BIT);
   delay_us(45);
   } 
   else
   {
    W1_DDR |= 1<<W1_BIT; 
    W1_PORT&= ~(1<<W1_BIT);
    delay_us(120);
    W1_DDR &= ~(1<<W1_BIT);
    W1_PORT&= ~(1<<W1_BIT);
    delay_us(1); 
   } 
  
 } 
 
}

//   bit   1-wire
unsigned char w1_receive_bit()
{
 unsigned char data=0;
 
 W1_DDR |= 1<<W1_BIT; 
 W1_PORT&= ~(1<<W1_BIT);
 delay_us(15);
 W1_DDR &= ~(1<<W1_BIT);
 W1_PORT&= ~(1<<W1_BIT);

  if((W1_PIN & (1<<W1_BIT)) == 0x00)
  data = 0;
  else
  data = 1; 
  delay_us(45); 
 
 return data;
}

//      1-wire
unsigned char w1_receive_byte()
{
 unsigned char data_temp=0;
 unsigned char i = 0;

 for(i = 0; i < 8; i++)
 { 
  data_temp >>= 1;
  if(w1_receive_bit()) // 1,      1
  data_temp |= 0x80;
 }
 return data_temp;
}

unsigned char crc8(unsigned char d,unsigned char crc_s)
{
unsigned char feedback_bit=0;
unsigned char  bit_counter = 8;

do
 {
   feedback_bit = (crc_s ^ d )& 0x01;
    if ( feedback_bit == 0x01 ) crc_s = crc_s ^ 0x18;
    crc_s = (crc_s >> 1) & 0x7F;  
    if ( feedback_bit == 0x01 ) crc_s = crc_s | 0x80;
    d = d >> 1;
    bit_counter--;
 }  while (bit_counter > 0);
  return crc_s;
}

void ds18b20_convert_t_rom() //    
{
 
   if(w1_find()==1)//    
 { 
 w1_sendcmd(0xcc);// ROM ,  ,        
               
 w1_sendcmd(0x44);//    
 w1_find();//  Presence  Reset
 }
 
}

int ds18b20_result_t_rom()  //   
{
   char i=0;
 int temp = 0;   
 unsigned char crc=0;
 unsigned char dat[9];
   if(w1_find()==1)//    
 {   
  w1_sendcmd(0xcc);// ROM ,  ,        
  
 w1_sendcmd(0xbe);//  ( 18b20     )    

  crc=0;
  for(i=0;i<9;i++)
  {
   dat[i] = w1_receive_byte();   // 
   if(i<8){ crc = crc8(dat[i],crc); } //  
  }
  if(crc!=dat[8]){return ERROR_CRC;}  //   
 //    
 temp = dat[1];
 temp = temp<<8;
 temp |= dat[0];
 //  
 temp = temp*0.625; //0.625     
 } 
 if(temp>1250||temp<-550){return ERROR_TMP;}
  return temp;
}

//     18b20   
int temp_18b20(unsigned char pinn)
{
 int temper=0;
 ds18b20_pin(pinn);
 ds18b20_convert_t_rom();
 
 while(!ds18b20_convert_ok(pinn)){;};
     
 temper = ds18b20_result_t_rom();
 // 
 return temper;
}

//     18b20      float 0.0
float Ftemp_18b20(unsigned char pinn)
{
 float temper=0;
 
 temper = temp_18b20(pinn);
 //   float
 return temper/10;
}